#ifndef GUARD_dirwnd_h
#define GUARD_dirwnd_h
// Sagua is music player.
//
// Orignal widget first appear in 'Air' project(http://air.kldp.net),
// and this is compact type. 
//
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if HAVE_EXT_HASH_SET && __GNUC__ >= 3
#include <ext/hash_set>
#else
#include <set>
#endif

#include <string>

#include <gtkmm/window.h>
#include <gtkmm/treemodel.h>
#include <gtkmm/treestore.h>
#include <gtkmm/treeview.h>
#include <gtkmm/menu.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/separator.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/button.h>
#include <gtkmm/stock.h>
#include <gtkmm/dialog.h>

class DirBrowseWnd : public Gtk::ScrolledWindow
{
    public:
	DirBrowseWnd(const int& = 12, const bool& = true, const std::string& = "");
	virtual ~DirBrowseWnd();

	const std::string& SelectedDirectory() { return selectedDirectory_; }
	const std::string& SelectedDirectoryFromIter(); 

	sigc::signal<void> signalSelected() { return signalSelected_; }

    private:
	// External container to check if path is in treestore.
	// If you can you hash set, try hash set. (not standard STL yet)
#if HAVE_EXT_HASH_SET && __GNUC__ >= 3
	struct StringHash
	{
	    size_t operator()(const std::string& id) const
	    {
		const std::string::size_type size = id.size();
		size_t h = 0;
		for(std::string::size_type i = 0; i < size; i++)
		{
		    h = 5 * h + id[i];
		}

		return h;
	    }
	};
	
	__gnu_cxx::hash_set<std::string, StringHash> pathInStore_;

	void ResetPathSet()
	{
	    __gnu_cxx::hash_set<std::string, StringHash>().swap(pathInStore_);
	}

#else	
	std::set<std::string> pathInStore_;

	void ResetPathSet()
	{
	    std::set<std::string>().swap(pathInStore_);
	}
	
#endif
    
	class ModelColumns : public Gtk::TreeModel::ColumnRecord
	{
	    public:
		ModelColumns() { add(colName_); add(colPath_); }

		// colName_ is display element, so i'll try it Glib::ustring.
		Gtk::TreeModelColumn<Glib::ustring> colName_;
		Gtk::TreeModelColumn<std::string> colPath_;
	};

	ModelColumns column_;	
    
	Gtk::TreeView treeView_;

	Gtk::Menu menuPopup_;	
	
	// selectedDirectory for return
	static std::string selectedDirectory_;

	std::string currentPath_;
	std::string fullPath_;

	Glib::RefPtr<Gtk::TreeStore> treeModel_;

	sigc::signal<void> signalSelected_;

	bool displayWatch_;
	
	virtual void on_show();

	void OnBtnPressEvent(GdkEventButton*);
	
	void OnRowActivated(const Gtk::TreeModel::Path&, Gtk::TreeViewColumn*);
	void OnRowExpanded(const Gtk::TreeModel::iterator&, const Gtk::TreeModel::Path&);

	void ExpandCurrentDir();

	void AddRootDirectory()
	{
	    Gtk::TreeModel::Row parent = *(treeModel_->append());
	    parent[column_.colName_] = "/";
	    parent[column_.colPath_] = "/";

	    treeModel_->append(parent.children());
	}

	void OnMenuRefresh();

};

#endif
