#ifndef GUARD_misc_h
#define GUARD_misc_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <vector>
#include <map>
#include <set>
#include <string>
#include <cctype>

#include <glibmm.h>

std::string GetExt(const std::string&);

struct ToLower: std::unary_function<char, void>
{
    void operator()(char& p) const
    {
	p = std::tolower(p);
    }
};

struct ToUpper: std::unary_function<char, void>
{
    void operator()(char& p) const
    {
	p = std::toupper(p);
    }
};

struct IsCharSpace: std::unary_function<char, bool>
{
    bool operator()(const char& p) const
    {
	return isspace(p);	
    }
};    

struct IsCharEqual: std::unary_function<char, bool>
{
    bool operator()(const char& p) const
    {
	if(p == '=')
	    return true;
	else
	    return false;
    }
};

inline bool StringIsSpace(const Glib::ustring& str)
{
    typedef Glib::ustring::const_iterator iter;
   
    iter stringEnd = str.end();

    for(iter eachCharacter = str.begin(); eachCharacter != stringEnd; ++eachCharacter)
    {
	if(!isspace(static_cast<char>(*eachCharacter)))
	    return false;	
    }

    return true;
}

inline bool IsDirectory(const std::string& p)
{
    return Glib::file_test(p, Glib::FILE_TEST_IS_DIR);
}

// If path is executable, return true
// Also, can test with Glib::file_test
inline bool IsExecutable(const std::string& p)
{
    return (access(p.c_str(), X_OK) == 0);
}

// If path is readable, return true
inline bool IsReadable(const std::string& p)
{
    return (access(p.c_str(), R_OK) == 0);
}

// If character is slash, return true 
struct IsCharSlash: std::unary_function<char, bool>
{
    bool operator()(const char& p) const
    {
	if(p == '/')
	    return true;
	else
	    return false;    
    }
};

Glib::ustring StringToUTF8(const Glib::ustring&, const std::string&, const std::string& = "eucKR");

void SplitEachDirToVector(const std::string&, std::vector<std::string>&);

void GetFilenameFromDirToVector(const std::string&, std::vector<std::string>&, bool = true);

bool ConfigfileToMap(const std::string&, std::map<std::string, std::string>&, const bool& = true, const bool& = true);

#endif
