#ifndef GUARD_player_h
#define GUARD_player_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <map>
#include <string>

#include "fixedallocator.h"
#include "misc.h"

class MainManager;

class Player 
{
    public:
	Player() {}
	virtual ~Player() {} 

	virtual bool Init(const std::string& filename) = 0;
	virtual void Start() = 0;
	virtual void Stop() = 0;
	virtual void Seek(const unsigned long&) = 0;
	virtual long int CurrentTime() = 0;

    protected:
	virtual void Clean() = 0;
	virtual void Reset() = 0;
};

class PlayerFactory : public SmallObject
{
    public:
	bool Register(const std::string& id, MainManager* mainManager, std::auto_ptr<Player> (*creator)(MainManager&))
	{
	    mainManager_ = mainManager;

	    return callbacks_.insert( callback::value_type(id, creator) ).second;
	}

	bool Unregister(const std::string& id)
	{
	    return callbacks_.erase(id) == 1;
	}

	bool Exist(const std::string& filename) const
	{
	    callback::const_iterator i = callbacks_.find(GetExt(filename));
	    
	    if(i != callbacks_.end())
		return true;
	    else
		return false;	
	}
	
	std::auto_ptr<Player> Create(const std::string& filename)
	{
	    callback::const_iterator i = callbacks_.find(GetExt(filename));

	    if(i != callbacks_.end())
		return (i->second)(*mainManager_);
	    else
		return std::auto_ptr<Player>();	
	}

    private:
	typedef std::map<std::string, std::auto_ptr<Player> (*)(MainManager&)> callback;
	callback callbacks_;
	MainManager* mainManager_;
};

#endif
