#ifndef GUARD_player_mp3_h
#define GUARD_player_mp3_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <unistd.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>

#include <vector>
#include <string>

#include <glibmm.h>

#include <mad.h>
#include "player.h"

class MainManager;

namespace MAD {

    struct buffer {
	unsigned char* fdm;
	unsigned char* data;
	unsigned long length;

	int fd;
	int eof;

	struct stat stat;
    };

    struct Timer {
	mad_timer_t playTimer_;
	unsigned long playFrameCount_;
	long playTimeSeconds_;
    };

    enum DecodeState {
	UNKNOWN = -1,
	PLAY = 0,
	STOP = 1,
	SEEK = 2,
	SEEK_REWIND = 3 
    };

    class MP3Player : public Player 
    {
	public:
	    MP3Player(MainManager&);
	    virtual ~MP3Player();

	    virtual bool Init(const std::string&);

	    virtual void Start();

	    virtual void Stop(); 
	    
	    virtual void Seek(const unsigned long&);

	    virtual long int CurrentTime() 
	    { 
		return timer_().playTimeSeconds_;
	    }

	private:
	    MP3Player& operator=(const MP3Player&);
	    MP3Player(const MP3Player&);
	
	    static MainManager* mainManager_;
	    
	    Glib::Dispatcher signalFinish_;
	    static Glib::Dispatcher signalTimerUpdate_;

	    static std::vector<double> bufSample_;

	    static std::string& filename_()
	    {
		static std::string filename__;

		return filename__;
	    }

	    struct mad_decoder decoder_;
	    struct buffer buf_;

	    static DecodeState& decodeState_()
	    {
		static DecodeState decodeState__;

		return decodeState__;
	    }

	    static unsigned int& sampleRate_()
	    {
		static unsigned int sampleRate__;

		return sampleRate__;
	    }

	    static Timer& timer_()
	    {
		static Timer timer__;

		return timer__;
	    }

	    static long& seekSeconds_()
	    {
		static long seekSeconds__;

		return seekSeconds__;
	    }

	    virtual void Clean()
	    {
	    	mad_decoder_finish(&decoder_);

		if(buf_.fdm)
		    munmap(buf_.fdm, buf_.stat.st_size);

		buf_.fdm = 0;
		
		if(!buf_.eof)
		    buf_.data = 0;    

		if(buf_.data) {
		    free(buf_.data);
		    buf_.data = 0;
		}
		
		if(buf_.fd != -1)
		    close(buf_.fd);
	    }
	    
	    virtual void Reset();

	    static enum mad_flow Input(void*, struct mad_stream*);
	    static enum mad_flow Output(void*, struct mad_header const*, struct mad_pcm*);
	    static enum mad_flow Header(void*, struct mad_header const*);
	    static enum mad_flow Error(void*, struct mad_stream*, struct mad_frame*); 
    };

}

#endif
