#ifndef GUARD_player_wma_h
#define GUARD_player_wma_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// wma decoder came from bmp-wma plugin.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <vector>
#include <string>

#include <glibmm.h>

#include "player.h"

#include "../src/libffwma/avcodec.h"
#include "../src/libffwma/avformat.h"

class MainManager;

namespace FFMPEG {

    enum DecodeState {
	UNKNOWN = 0,
	PLAY = -1,
	STOP = -2,
	SEEK = -3
    };
    
    class WMAPlayer : public Player
    {
	public:
	    WMAPlayer(MainManager&);
	    virtual ~WMAPlayer();
	    
	    virtual bool Init(const std::string&);
	    virtual void Start();
	    virtual void Stop();
	    virtual void Seek(const unsigned long&);
	    virtual long int CurrentTime();

	private:
	    WMAPlayer& operator=(const WMAPlayer&);
	    WMAPlayer(const WMAPlayer&);

	    Glib::Dispatcher signalFinish_;
	    Glib::Dispatcher signalTimerUpdate_;

	    Glib::Mutex avMutex_;

	    DecodeState decodeState_;

	    long int currentSeconds_;
	    unsigned long int seekSeconds_;
	    const int bufSize_;
	    int sampleRate_;
	    int index_;
	    
	    bool validFile_;

	    std::vector<double> buf_;

	    MainManager& mainManager_;

	    AVCodec* avCodec_;
	    AVPacket avPacket_;
	    AVCodecContext* avCodecContext_;
	    AVFormatContext* avFormatContext_;

	    uint8_t* outbuf_;
	    uint8_t* s_outbuf_;
	
	    virtual void Clean();
	    virtual void Reset();

    };
}

#endif
