#ifndef GUARD_playlist_h
#define GUARD_playlist_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <string>
#include <queue>

#include <gtkmm/treeview.h>
#include <gtkmm/treemodel.h>
#include <gtkmm/liststore.h>
#include <gtkmm/menu.h>

class MainWnd;
class Tag;
class TagFactory;

class Playlist : public Gtk::TreeView
{
    public:
	Playlist(MainWnd& mainWnd, const std::string&, const unsigned int& = 470);
	virtual ~Playlist();

	void Add(const std::string&);
	
	void RemoveAll() 
	{   
	    OnMenuRemoveAll(); 
	}
	
	std::string Get();
	std::string GetNext();
	std::string GetPrev();
	std::string GetRandom();
	void ForceUpdatePage();

	void ShowCurrentFileInfoDlg();

    private:
	class ModelColumns : public Gtk::TreeModel::ColumnRecord
	{
	    public:
		ModelColumns()
		{
		    add(tagParsed_);
		    add(id_);
		    add(filename_);
		    add(title_);
		    add(time_);
		    add(timeSeconds_);
		}

		Gtk::TreeModelColumn<bool> tagParsed_;
		Gtk::TreeModelColumn<unsigned int> id_;
		Gtk::TreeModelColumn<std::string> filename_;
		Gtk::TreeModelColumn<Glib::ustring> title_;
		Gtk::TreeModelColumn<Glib::ustring> time_;
		Gtk::TreeModelColumn<unsigned int> timeSeconds_;
	};

	struct EachRow 
	{
	    Gtk::TreeModel::iterator iter_;
	    bool tagParsed_;
	    Glib::ustring title_;
	    Glib::ustring time_;
	    unsigned int timeSeconds_;
	};

	std::string listId_;

	ModelColumns columns_;

	Gtk::Menu menuPopup_;
	Gtk::TreeModel::iterator pos_;

	MainWnd& mainWnd_;

	Glib::Thread* tagParsingThread_;
	Glib::Mutex tagParsingMutex_;
	Glib::Dispatcher signalRowToFill_;
	std::queue<EachRow> rowQueue_;
	void RowFill(); 

	unsigned int count_;
	unsigned int tagProcessCount_;

	bool inTagParsing_;
	bool stopTagParsing_;

	Glib::RefPtr<Gtk::ListStore> treeModel_;
	
	std::auto_ptr<TagFactory> tagFactory_;
	
	virtual bool on_button_press_event(GdkEventButton*);
	virtual void on_row_activated(const Gtk::TreeModel::Path&, Gtk::TreeViewColumn*);
	virtual void on_set_scroll_adjustments(Gtk::Adjustment*, Gtk::Adjustment*);
	virtual void on_size_allocate(Gtk::Allocation&);

	int FilenameCompare(const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b)
	{
	    std::string strA = (*a)[columns_.filename_];
	    std::string strB = (*b)[columns_.filename_];

	    std::string aPath = Glib::path_get_dirname(strA);
	    std::string bPath = Glib::path_get_dirname(strB);
	    
	    // If files in same directory, sort them
	    // or sort for directory.	
	    if(aPath == bPath)
		return strcmp(strA.c_str(), strB.c_str());
	    else
		return strcmp(aPath.c_str(), bPath.c_str());
	}

	void MoveCursorAndScroll(const Gtk::TreeModel::iterator& iter)
	{
	    Gtk::TreeModel::Path path;
	    if(iter) {
		path = Gtk::TreeModel::Path(iter);
	    } else {
		path = Gtk::TreeModel::Path("0");
	    }
	    set_cursor(path);
	    scroll_to_row(path, 0.5);
	}

	std::string GetFilenameFromTreeiter(Gtk::TreeModel::iterator&); 
	void ParseTag(Gtk::TreeModel::iterator&, const bool&);
	void GetInfoFromTag(Gtk::TreeModel::iterator&);

	void ShowFileInfoDlg(const std::string&);

	void OnMenuRemove();
	void OnMenuRemoveAll();
	void OnMenuSortFilename();
	void OnMenuInfo();

	void OnVScrolled();

	void LoadList();
	void SaveList();

	void StopThread();

};

#endif
