#ifndef GUARD_preference_h
#define GUARD_preference_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <map>
#include <string>

class Preference;
class PrefWnd;

class Preference {
    public:
	Preference();
	~Preference();

	friend void operator<<(Preference&, const PrefWnd&);

	const bool& SuccessLoadConfig() const { return successLoadConfig_; }

	const std::string& GeneralTrayIconAction() const { return generalTrayIconAction_; }
	const bool& GeneralLeaveOnTrayWhenIconify() const { return generalLeaveOnTrayWhenIconify_; }

	const std::string& AudioDriver() const { return audioDriver_; }
	void SetDefaultAudioDriver() { audioDriver_ = "default"; }
	
	const std::string& AudioAlsa5Card() const { return audioAlsa5Card_; }
	const std::string& AudioAlsa5Dev() const { return audioAlsa5Dev_; }

	const std::string& AudioAlsa9Dev() const { return audioAlsa9Dev_; }

	const std::string& AudioOssDsp() const { return audioOssDsp_; }
	const std::string& AudioSunDev() const { return audioSunDev_; }
	const std::string& AudioAixsDev() const { return audioAixsDev_; }

	const double& EfxStereoEnhanceAmount() const { return efxStereoEnhanceAmount_; }
	const double& EfxRoomAmount() const { return efxRoomAmount_; }
	const double& EfxRoom2Amount() const { return efxRoom2Amount_; }
	const double& EfxHarmonyAmount() const { return efxHarmonyAmount_; }
	const double& EfxRhythmeAmount() const { return efxRhythmeAmount_; }
	const double& EfxDistortionAmount() const { return efxDistortionAmount_; }

	const float* EfxGraphicEQGain() const { return efxGraphicEQGain_; }

	const bool& Effect() const { return effect_; }    
	const bool& EfxBassBoost() const { return efxBassBoost_; }
	const bool& EfxBassBoost2() const { return efxBassBoost2_; }
	const bool& EfxStereoEnhance() const { return efxStereoEnhance_; }
	const bool& EfxTrebleBoost() const { return efxTrebleBoost_; }
	const bool& EfxRoom() const { return efxRoom_; }
	const bool& EfxRoom2() const { return efxRoom2_; }
	const bool& EfxHarmony() const { return efxHarmony_; }
	const bool& EfxRhythme() const { return efxRhythme_; }
	const bool& EfxDistortion() const { return efxDistortion_; }
	const bool& EfxGraphicEQ() const { return efxGraphicEQ_; }

    private:	
	std::string generalTrayIconAction_; 
    
	std::string audioDriver_;
	
	std::string audioAlsa5Card_;
	std::string audioAlsa5Dev_;

	std::string audioAlsa9Dev_;

	std::string audioOssDsp_;
	std::string audioSunDev_;
	std::string audioAixsDev_;
    
	double efxStereoEnhanceAmount_;
	double efxRoomAmount_;
	double efxRoom2Amount_;
	double efxHarmonyAmount_;
	double efxRhythmeAmount_;
	double efxDistortionAmount_;

	float efxGraphicEQGain_[11];
	
	bool successLoadConfig_;
	bool generalLeaveOnTrayWhenIconify_;
	bool effect_;	
	bool efxBassBoost_;
	bool efxBassBoost2_;
	bool efxStereoEnhance_;
	bool efxTrebleBoost_;     
	bool efxRoom_;
	bool efxRoom2_;
	bool efxHarmony_;
	bool efxRhythme_;
	bool efxDistortion_;
	bool efxGraphicEQ_;

	bool LoadConfig();
	void AssignConfig(bool&, const std::string&, const std::map<std::string, std::string>&);
	void AssignConfig(double&, const std::string&, const std::map<std::string, std::string>&);
	void AssignConfig(float&, const std::string&, const std::map<std::string, std::string>&);
	void AssignConfig(std::string&, const std::string&, const std::map<std::string, std::string>&);
	
};

#endif
