#ifndef GUARD_tag_h
#define GUARD_tag_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <string>

#include <glibmm.h>

#include "fixedallocator.h"
#include "misc.h"

class Tag 
{ 
    public:
	// Gtk Scale's min is 0 and max must be bigger than min.
	// So totalTimeSeconds' initial value is 1, not 0.
	Tag(): totalTimeSeconds_(1) {}
	virtual ~Tag() {}

	const Glib::ustring& GetTitle()
	{
	    if(title_.empty())
		ProcessTitle();

	    return title_;
	}
	
	const Glib::ustring& GetAlbum()
	{
	    if(album_.empty())
		ProcessAlbum();

	    return album_;
	}

	const Glib::ustring& GetGenre()
	{
	    if(genre_.empty())
		ProcessGenre();

	    return genre_;
	}
	
	const Glib::ustring& GetArtist()
	{
	    if(artist_.empty())
		ProcessArtist();

	    return artist_;
	}
	
	const Glib::ustring& GetComment() 
	{
	    if(comment_.empty())
		ProcessComment();

	    return comment_;	
	}

	const Glib::ustring& GetYear() 
	{
	    if(year_.empty())
		ProcessYear();
		
	    return year_;
	}
	
	const Glib::ustring& GetTrack() 
	{
	    if(track_.empty())
		ProcessYear();

	    return track_;
	}

	const Glib::ustring& GetTotalTime()
	{
	    if(totalTime_.empty())
		ProcessTotalTime();

	    return totalTime_;
	}

	const unsigned long& GetTotalTimeSeconds()
	{
	    if(totalTimeSeconds_ == 1)
		ProcessTotalTime();

	    return totalTimeSeconds_;	
	}

    protected:
	// To display in window, i'll use Glib::ustring instead std::string
	Glib::ustring title_, artist_, album_, track_, year_;
	Glib::ustring genre_, comment_;
	Glib::ustring totalTime_;
	unsigned long totalTimeSeconds_;

	virtual void ProcessAll() = 0;
	virtual void ProcessYear() = 0;
	virtual void ProcessTrack() = 0;
	virtual void ProcessTitle() = 0;
	virtual void ProcessAlbum() = 0;
	virtual void ProcessGenre() = 0;
	virtual void ProcessArtist() = 0;
	virtual void ProcessComment() = 0;
	virtual void ProcessTotalTime() = 0;
	

};

class TagFactory : public SmallObject
{
    public:
	bool Register(const std::string& id, std::auto_ptr<Tag> (*creator)(const std::string&))
	{
	    return callbacks_.insert( callback::value_type(id, creator) ).second;
	}

	bool Unregister(const std::string& id)
	{
	    return callbacks_.erase(id) == 1;
	}

	bool Exist(const std::string& filename)
	{
	    callback::const_iterator i = callbacks_.find(GetExt(filename));
	    
	    if(i != callbacks_.end())
		return true;
	    else
		return false;	
	}
	
	std::auto_ptr<Tag> Create(const std::string& filename)
	{
	    callback::const_iterator i = callbacks_.find(GetExt(filename));

	    if(i != callbacks_.end())
		return (i->second)(filename);
	    else
		return std::auto_ptr<Tag>();	
	}

    private:
	typedef std::map<std::string, std::auto_ptr<Tag> (*)(const std::string&)> callback;
	callback callbacks_;
};


#endif
