#ifndef GUARD_trace_h
#define GUARD_trace_h
// Sagua trace object.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <iostream>
#include <string>

namespace Tracer {

    inline bool& activeTrace()
    {
	static bool activeTrace_;

	return activeTrace_;
    }

#ifdef ENABLE_DEBUGTRACE 

    class Trace {
	public:
	    Trace(const char* name, std::ostream& os = std::cout)
	    : os_(os)
	    {
		if(activeTrace()) {
		    os_ << ">> Enter : " << name << std::endl;
		    functionName_ = std::auto_ptr<std::string>(new std::string(name));
		}
	    }

	    ~Trace()
	    {
		if(activeTrace()) {
		    os_ << "<< Leave : " << *functionName_ << std::endl;
		}
	    }

	    void Debug(const char* msg)
	    {
		if(activeTrace()) {
		    os_ << '[' << *functionName_ << "] " << msg << std::endl;
		}
	    }

	private:
	    std::auto_ptr<std::string> functionName_;
	    std::ostream& os_;
	    
    };
    
#endif

#ifdef DISABLE_DEBUGTRACE

    class Trace {
	public:
	    Trace(const char* name, std::ostream& os = std::cout) {}
	    ~Trace() {}
	    void Debug(const char* msg) {}
    };

#endif    
    
}

#endif
