// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <string>
#include <vector>

#include <gtkmm/scrolledwindow.h>

#include "../include/dirplaywin.h"

#include "../include/mainwin.h"
#include "../include/player.h"
#include "../include/playlist.h"
#include "../include/dirbrowsewin.h"
#include "../include/misc.h"

class DirPlayWnd::DirPlayWndImpl
{
    public:
	DirPlayWndImpl(MainWnd&, DirPlayWnd&, const unsigned int&);
	~DirPlayWndImpl() {};

	DirBrowseWnd dirBrowseWnd_;
	Gtk::ScrolledWindow scrolledWindow_;
	Playlist playlist_;
	MainWnd& mainWnd_;

	void OnSelected();
    
};

DirPlayWnd::DirPlayWnd(MainWnd& mainWnd, const unsigned int& titleWidth)
: impl_(new DirPlayWndImpl(mainWnd, *this, titleWidth))
{}

DirPlayWnd::~DirPlayWnd()
{}

Playlist* const DirPlayWnd::GetPlaylist()
{
    return &(impl_->playlist_);
}

DirPlayWnd::DirPlayWndImpl::DirPlayWndImpl(MainWnd& mainWnd, DirPlayWnd& dirPlayWnd, const unsigned int& titleWidth)
    : dirBrowseWnd_(0, false, mainWnd.GetLastBrowsedDirectory()),
    playlist_(mainWnd, "dir", titleWidth),
    mainWnd_(mainWnd)
{
    dirPlayWnd.add1(dirBrowseWnd_);

    scrolledWindow_.add(playlist_);
    scrolledWindow_.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    dirPlayWnd.add2(scrolledWindow_);

    dirBrowseWnd_.signalSelected().connect(
	    sigc::mem_fun(*this, &DirPlayWndImpl::OnSelected) );

    dirPlayWnd.set_position(229);

    dirPlayWnd.show_all_children();
}

void DirPlayWnd::DirPlayWndImpl::OnSelected()
{
    playlist_.RemoveAll();

    mainWnd_.LogBrowsedDirectory(dirBrowseWnd_.SelectedDirectory());
    
    std::vector<std::string> selectedFilesVector;
    GetFilenameFromDirToVector(dirBrowseWnd_.SelectedDirectory(), selectedFilesVector, false);

    typedef std::vector<std::string>::const_iterator iter;
    iter end = selectedFilesVector.end();

    for(iter i = selectedFilesVector.begin(); i != end; ++i)
    {
	if(mainWnd_.GetPlayerFactory()->Exist(*i))
	    playlist_.Add(*i);
    }

    playlist_.ForceUpdatePage();

}

