// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "../include/effect.h"

// Apply clipping.
void Effect::Clip(double& inL, double& inR)
{
    clipAbsInL() = fabs(inL);
    clipAbsInR() = fabs(inR);

    if(clipAbsInL() > 1.0) {
	if(clipAbsInL() > peakSample()[0]) { 
	    peakSample()[1] = peakSample()[0];
	    peakSample()[0] = clipAbsInL();
	    feedGain() = 20.0;
	    feedGain() *= log((peakSample()[0] + peakSample()[1]) * 0.5);
	} 
	if(inL > 1.0)
	    inL = 1.0;
	else if(inL < -1.0)
	    inL = -1.0;    
    }

    if(clipAbsInR() > 1.0) {
	if(clipAbsInR() > peakSample()[0]) { 
	    peakSample()[1] = peakSample()[0];
	    peakSample()[0] = clipAbsInR();
	    feedGain() = 20.0;
	    feedGain() *= log((peakSample()[0] + peakSample()[1]) * 0.5);
	}
	if(inR > 1.0)
	    inR = 1.0;
	else if(inR < -1.0)
	    inR = -1.0;    
    }

}

// Add diffrence of stereo signal.
void Effect::StereoEnhance(double& left, double& right, const float& wide)
{
    double stereoEnhanceMono = left;
    stereoEnhanceMono += right;
    stereoEnhanceMono *= 0.5;
    
    double stereoEnhanceDiffLeft = left;
    stereoEnhanceDiffLeft -= stereoEnhanceMono;
    
    stereoEnhanceDiffLeft *= wide;
    left += stereoEnhanceDiffLeft;
    right -= stereoEnhanceDiffLeft;

    Clip(left, right);

}

// Gain is in dB.
void Effect::Amplify(double& sampleL, double& sampleR, const float& gain)
{
    if(gain == 0.0)
	return;

    if(amplifyGain() != gain || amplifyRatio() == 0.0) {
	amplifyGain() = gain;
	amplifyRatio() = pow(10.0, gain * 0.05);
    }

    sampleL *= amplifyRatio();
    sampleR *= amplifyRatio();

    Clip(sampleL, sampleR);

}

// Apply Low Shelf EQ.
void Effect::BassBoost(const int& sampleRate, double& sampleL, double& sampleR)
{
    if(Bass100.SampleRate() != sampleRate || Bass100.Freq() != 100)
    {	
	Bass100.Calc(sampleRate, 100, 12.0f, 0.60295);
    }

    Bass100.Apply(sampleL, BassBoost_L[0]);
    Bass100.Apply(sampleR, BassBoost_R[0]);

    Clip(sampleL, sampleR);

    EQ(Effect::Bass50, sampleRate, sampleL, sampleR, 50, 3.0f, BassBoost_L[1], BassBoost_R[1], 1.4f);

}

void Effect::BassBoost2(const int& sampleRate, double& sampleL, double& sampleR)
{
    if(Bass2_80.SampleRate() != sampleRate) {
	
	Bass2_80.Calc(sampleRate, 80, 12.0f, 0.76077);
	Bass2_199.Calc(sampleRate, 199, 5.5f, 0.33374);
	Bass2_1309.Calc(sampleRate, 1309, 9.0f, 0.25842);

    }

    Bass2_199.Apply(sampleL, BassBoost2_L[0]);
    Bass2_199.Apply(sampleR, BassBoost2_R[0]);
    
    Bass2_1309.Apply(sampleL, BassBoost2_L[1]);
    Bass2_1309.Apply(sampleR, BassBoost2_R[1]);
    
    Bass2_80.Apply(sampleL, BassBoost2_L[2]);
    Bass2_80.Apply(sampleR, BassBoost2_R[2]);
    
    Clip(sampleL, sampleR);

}

// Apply High Shelf EQ, but also can increase high frequency noise.
void Effect::TrebleBoost(const int& sampleRate, double& sampleL, double& sampleR)
{
    if(Treble5k.SampleRate() != sampleRate || Treble5k.Freq() != 5000)
    {	
	Treble5k.Calc(sampleRate, 5000, 6.0f, 0.7);
    }

    Treble5k.Apply(sampleL, TrebleBoost_L);
    Treble5k.Apply(sampleR, TrebleBoost_R);

    Clip(sampleL, sampleR);

}

// Apply Peak EQ.
void Effect::EQ(BiquadCoeff& p, const int& sampleRate, double& sampleL, double& sampleR, const int& freq, const float& gain, Base& prevL, Base& prevR, const float& Q)
{
    if(gain == 0.0)
	return;

    if(p.SampleRate() != sampleRate || p.Freq() != freq)
    {
	p.Calc(sampleRate, freq, gain, Q);
    }
    
    p.Apply(sampleL, prevL);
    p.Apply(sampleR, prevR);

    Clip(sampleL, sampleR);

}

// Apply 11 Band Graphic EQ
void Effect::GraphicEQ(const int& sampleRate, double& sampleL, double& sampleR, const float* const parameter)
{
    if(EQ50.SampleRate() != sampleRate)
    {
	EQ50.Calc(sampleRate, 50, parameter[0], 1.0);
	EQ100.Calc(sampleRate, 100, parameter[1], 1.0);
	EQ200.Calc(sampleRate, 200, parameter[2], 1.0);
	EQ400.Calc(sampleRate, 400, parameter[3], 1.0);
	EQ800.Calc(sampleRate, 800, parameter[4], 1.0);
	EQ1500.Calc(sampleRate, 1500, parameter[5], 1.0);
	EQ3k.Calc(sampleRate, 3000, parameter[6], 1.0);
	EQ5k.Calc(sampleRate, 5000, parameter[7], 1.0);
	EQ7k.Calc(sampleRate, 7000, parameter[8], 1.0);
	EQ10k.Calc(sampleRate, 10000, parameter[9], 1.0);
	EQ15k.Calc(sampleRate, 15000, parameter[10], 1.0);
    }

    EQ50.Apply(sampleL, GraphicEQ_L[0]);
    EQ50.Apply(sampleR, GraphicEQ_R[0]);
    
    EQ100.Apply(sampleL, GraphicEQ_L[1]);
    EQ100.Apply(sampleR, GraphicEQ_R[1]);

    EQ200.Apply(sampleL, GraphicEQ_L[2]);
    EQ200.Apply(sampleR, GraphicEQ_R[2]);

    EQ400.Apply(sampleL, GraphicEQ_L[3]);
    EQ400.Apply(sampleR, GraphicEQ_R[3]);

    EQ800.Apply(sampleL, GraphicEQ_L[4]);
    EQ800.Apply(sampleR, GraphicEQ_R[4]);

    EQ1500.Apply(sampleL, GraphicEQ_L[5]);
    EQ1500.Apply(sampleR, GraphicEQ_R[5]);

    EQ3k.Apply(sampleL, GraphicEQ_L[6]);
    EQ3k.Apply(sampleR, GraphicEQ_R[6]);

    EQ5k.Apply(sampleL, GraphicEQ_L[7]);
    EQ5k.Apply(sampleR, GraphicEQ_R[7]);
    
    EQ7k.Apply(sampleL, GraphicEQ_L[8]);
    EQ7k.Apply(sampleR, GraphicEQ_R[8]);

    EQ10k.Apply(sampleL, GraphicEQ_L[9]);
    EQ10k.Apply(sampleR, GraphicEQ_R[9]);
    
    EQ15k.Apply(sampleL, GraphicEQ_L[10]);
    EQ15k.Apply(sampleR, GraphicEQ_R[10]);
    
    Clip(sampleL, sampleR);
}

// Apply Low Pass Filter.
Effect::Stereo Effect::LPF(BiquadCoeff& p, const int& sampleRate, double& sampleL, double& sampleR, const int& freq, const double& gain, Base& prevL, Base& prevR)
{
    if(p.SampleRate() != sampleRate || p.Freq() != freq)
    {
	p.Calc(sampleRate, freq, gain, 0.0);
    }

    return Stereo( p.Return(sampleL, prevL), p.Return(sampleR, prevR) );
}

// Apply High Pass Filter.
Effect::Stereo Effect::HPF(BiquadCoeff& p, const int& sampleRate, double& sampleL, double& sampleR, const int& freq, const double& gain, Base& prevL, Base& prevR)
{
    if(p.SampleRate() != sampleRate || p.Freq() != freq)
    {
	p.Calc(sampleRate, freq, gain, 0.0);
    }

    return Stereo( p.Return(sampleL, prevL), p.Return(sampleR, prevR) );
}

// Adjust gain accoding to average of signal.
void Effect::Rhythme(double& sampleL, double& sampleR, const double& mount)
{
    if(prevCount() != 18) {
	prevCount()++;
	return;
    } else { 
	// if prevCount() == 18
	double avg = sqrt(sampleL * sampleR);
	// To compute with variable 'feedGain_', it is negative value;

	if(feedRhythme() < mount && feedRhythme() > -mount + 0.2 && prevAvg()[0] < avg) { 
	    if(prevAvg()[0] * avg < 0.0)
		feedRhythme() = 0.0;
	    if(prevAvg()[1] < prevAvg()[0])
		feedRhythme() -= 0.2;
	    else 
		feedRhythme() -= 0.1;
	} else if(feedRhythme() < mount - 0.2 && feedRhythme() > -mount && prevAvg()[0] > avg) { 
	    if(prevAvg()[0] * avg < 0.0)
		feedRhythme() = 0.0;
	    if(prevAvg()[1] > prevAvg()[0])
		feedRhythme() += 0.2;
	    else  
		feedRhythme() += 0.1;
	}
	
	prevCount() = 0;
	prevAvg()[1] = prevAvg()[0];
	prevAvg()[0] = avg;

    }

    // There is no change in signal this time, there is no clipping.
}

// Add previous signal that LPF processed.
void Effect::Room(const int& sampleRate, double& sampleL, double& sampleR, const double& ab)
{
    Stereo filtered = LPF(LPF_, sampleRate, sampleL, sampleR, 5300, 0.75, Room_L, Room_R); 

    double abSquare = ab;
    abSquare *= ab;

    if(prevRoomCount() > 4)
	prevRoomCount() = 0;

    sampleL += prevLeftSample()[0] * ab * 0.75;	    
    sampleL += prevLeftSample()[1] * ab * 0.45;
    sampleL += prevLeftSample()[2] * abSquare * 0.60;
    sampleL += prevLeftSample()[3] * ab * 0.15;
    sampleL += prevLeftSample()[4] * abSquare * 0.30;

    sampleR += prevRightSample()[0] * ab * 0.75;	    
    sampleR += prevRightSample()[1] * ab * 0.45;
    sampleR += prevRightSample()[2] * abSquare * 0.60;
    sampleR += prevRightSample()[3] * ab * 0.15;
    sampleR += prevRightSample()[4] * abSquare * 0.30;

    prevLeftSample()[prevRoomCount()] = filtered.left;
    prevRightSample()[prevRoomCount()] = filtered.right;

    prevRoomCount()++;

    Clip(sampleL, sampleR);

}

void Effect::Room2(const int& sampleRate, double& sampleL, double& sampleR, const double& mount)
{
    if(Room2_62.SampleRate() != sampleRate) {

	Room2_62.Calc(sampleRate, 62, -8.0f, 0.51208);
	Room2_279.Calc(sampleRate, 279, 4.0f, 0.39539);
	Room2_13545.Calc(sampleRate, 13545, 4.0f, 0.37929);

	Room2_35.Calc(sampleRate, 35, -3.0f, 0.52443);
	Room2_852.Calc(sampleRate, 852, 8.0f + (10.0 * mount), 0.37624);
	Room2_3713.Calc(sampleRate, 3713, 12.0f, 0.39159);

    }

    Room2_279.Apply(sampleL, Room2_L[0]);
    Room2_279.Apply(sampleL, Room2_R[0]);

    Room2_62.Apply(sampleL, Room2_L[1]);
    Room2_62.Apply(sampleR, Room2_R[1]);

    Room2_13545.Apply(sampleL, Room2_L[2]);
    Room2_13545.Apply(sampleR, Room2_R[2]);

    Amplify(sampleL, sampleR, -3.0f);

    Stereo cross(sampleR, sampleL);
    
    Room2_852.Apply(cross.left, Room2_L[3]);
    Room2_852.Apply(cross.right, Room2_R[3]);

    Room2_35.Apply(cross.left, Room2_L[4]);
    Room2_35.Apply(cross.right, Room2_R[4]);

    Room2_3713.Apply(cross.left, Room2_L[5]);
    Room2_3713.Apply(cross.right, Room2_R[5]);

    Amplify(cross.left, cross.right, -15.0f - (10.0 * mount));

    sampleL += cross.left;
    sampleR += cross.right;
    
    Clip(sampleL, sampleR);

}

// Add previous signal that processed HPF on high average power.
void Effect::Harmony(const int& sampleRate, double& sampleL, double& sampleR, const double& ab)
{
    Stereo filtered = HPF(HPF_, sampleRate, sampleL, sampleR, 10000, 0.5, Harmony_L, Harmony_R); 
    
    if(prevHarmonyCount() > 2)
	prevHarmonyCount() = 0;

    prevLeftSample1()[prevHarmonyCount()] = filtered.left;
    prevRightSample1()[prevHarmonyCount()] = filtered.right;
    
    prevHarmonyCount()++;

    if(sqrt(sampleL * sampleR) > 0.75) {
	double abSquare = ab;
	abSquare *= ab;

	sampleL += prevLeftSample1()[2] * abSquare * 0.1;
	sampleL += prevLeftSample1()[1] * ab * 0.3;
	sampleL += prevLeftSample1()[0] * ab * 0.5;		

	sampleR += prevRightSample1()[2] * abSquare * 0.1;
	sampleR += prevRightSample1()[1] * ab * 0.3;
	sampleR += prevRightSample1()[0] * ab * 0.5;	
		
	Clip(sampleL, sampleR);
    }

}

void Effect::Distortion(double& sampleL, double& sampleR, const double& mount)
{
    double multiple;

    if(mount > 5.0)
	multiple = 0.2;
    else
	multiple = 1.0 / mount;
	
    sampleL = atan(sampleL * mount);
    sampleL *= multiple;	
    sampleR = atan(sampleR * mount);
    sampleR *= multiple;

}

void Effect::Process(const Preference& preference, const int& sampleRate, Stereo& orig)
{

    Effect::Amplify(orig.left, orig.right, -(feedGain() + feedRhythme()));

    if(preference.EfxBassBoost()) 
	Effect::BassBoost(sampleRate, orig.left, orig.right);

    if(preference.EfxBassBoost2())
	Effect::BassBoost2(sampleRate, orig.left, orig.right);

    if(preference.EfxTrebleBoost())
	TrebleBoost(sampleRate, orig.left, orig.right);
 
    if(preference.EfxGraphicEQ())
	Effect::GraphicEQ(sampleRate, orig.left, orig.right, preference.EfxGraphicEQGain());	
 
    if(preference.EfxStereoEnhance())
	Effect::StereoEnhance(orig.left, orig.right, preference.EfxStereoEnhanceAmount());

    if(preference.EfxRhythme())
	Effect::Rhythme(orig.left, orig.right, preference.EfxRhythmeAmount());
  
    if(preference.EfxRoom())
	Effect::Room(sampleRate, orig.left, orig.right, preference.EfxRoomAmount());

    if(preference.EfxRoom2())
	Effect::Room2(sampleRate, orig.left, orig.right, preference.EfxRoom2Amount());

    if(preference.EfxHarmony())
	Effect::Harmony(sampleRate, orig.left, orig.right, preference.EfxHarmonyAmount());

    if(preference.EfxDistortion())
	Effect::Distortion(orig.left, orig.right, preference.EfxDistortionAmount());
    
}

void Effect::Reset()
{
    peakSample()[0] = 1.0;
    peakSample()[1] = 1.0;
    feedGain() = 0.0;
    feedRhythme() = 0.0;
    prevAvg()[0] = 0.0;
    prevAvg()[1] = 0.0;
    prevCount() = 0;
    prevRoomCount() = 0;
    prevHarmonyCount() = 0;

    prevLeftSample()[0] = 0.0;
    prevLeftSample()[1] = 0.0;
    prevLeftSample()[2] = 0.0;
    prevLeftSample()[3] = 0.0;
    prevLeftSample()[4] = 0.0;
    
    prevRightSample()[0] = 0.0;
    prevRightSample()[1] = 0.0;
    prevRightSample()[2] = 0.0;
    prevRightSample()[3] = 0.0;
    prevRightSample()[4] = 0.0;

    prevLeftSample1()[0] = 0.0;
    prevLeftSample1()[1] = 0.0;
    prevLeftSample1()[2] = 0.0;
    prevRightSample1()[0] = 0.0;
    prevRightSample1()[1] = 0.0;
    prevRightSample1()[2] = 0.0;

    clipAbsInL() = 0.0;
    clipAbsInR() = 0.0;
    amplifyRatio() = 0.0;
    amplifyGain() = 0.0;

    BassBoost_L[0].Reset();
    BassBoost_L[1].Reset();
    BassBoost_R[0].Reset();
    BassBoost_R[1].Reset();

    BassBoost2_L[0].Reset();
    BassBoost2_L[1].Reset();
    BassBoost2_L[2].Reset();
    BassBoost2_R[0].Reset();
    BassBoost2_R[1].Reset();
    BassBoost2_R[2].Reset();

    TrebleBoost_L.Reset();
    TrebleBoost_R.Reset();

    GraphicEQ_L[0].Reset();
    GraphicEQ_L[1].Reset();
    GraphicEQ_L[2].Reset();
    GraphicEQ_L[3].Reset();
    GraphicEQ_L[4].Reset();
    GraphicEQ_L[5].Reset();
    GraphicEQ_L[6].Reset();
    GraphicEQ_L[7].Reset();
    GraphicEQ_L[8].Reset();
    GraphicEQ_L[9].Reset();
    GraphicEQ_L[10].Reset();
    
    GraphicEQ_R[0].Reset();
    GraphicEQ_R[1].Reset();
    GraphicEQ_R[2].Reset();
    GraphicEQ_R[3].Reset();
    GraphicEQ_R[4].Reset();
    GraphicEQ_R[5].Reset();
    GraphicEQ_R[6].Reset();
    GraphicEQ_R[7].Reset();
    GraphicEQ_R[8].Reset();
    GraphicEQ_R[9].Reset();
    GraphicEQ_R[10].Reset();

    Room_L.Reset();
    Room_R.Reset();

    Room2_L[0].Reset();
    Room2_L[1].Reset();
    Room2_L[2].Reset();
    Room2_L[3].Reset();
    Room2_L[4].Reset();
    Room2_L[5].Reset();
    Room2_R[0].Reset();
    Room2_R[1].Reset();
    Room2_R[2].Reset();
    Room2_R[3].Reset();
    Room2_R[4].Reset();
    Room2_R[5].Reset();

    Harmony_L.Reset();
    Harmony_R.Reset();

    Bass50.Reset();
    Bass100.Reset();
    Bass2_80.Reset();
    Bass2_199.Reset();
    Room2_35.Reset();
    Room2_62.Reset();
    Room2_279.Reset();
    Room2_852.Reset();
    Treble5k.Reset();
    Bass2_1309.Reset();
    Room2_3713.Reset();
    Room2_13545.Reset();
    LPF_.Reset();
    HPF_.Reset();

    EQ50.Reset();
    EQ100.Reset();
    EQ200.Reset();
    EQ400.Reset();
    EQ800.Reset();
    EQ1500.Reset();
    EQ3k.Reset();
    EQ5k.Reset();
    EQ7k.Reset();
    EQ10k.Reset();
    EQ15k.Reset();
    
}
