// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <iostream>
#include <fstream>

#include <cstdlib>
#include <string>
#include <vector>
#include <map>

#include <gdkmm.h>
#include <gtkmm/main.h>
#include <gtkmm/scrollbar.h>

#include <libintl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "../include/mainwin.h"
#include "../include/mainmanager.h"
#include "../include/prefwin.h"
#include "../include/player_mp3.h"
#include "../include/player_ogg.h"
#include "../include/player_wma.h"
#include "../include/playlist.h"
#include "../include/dirbrowsedlg.h"
#include "../include/dirplaywin.h"
#include "../include/misc.h"
#include "../include/trace.h"

std::auto_ptr<Player> CreateMP3Player(MainManager& mainManager)
{
    return std::auto_ptr<Player>(new MAD::MP3Player(mainManager));
}

std::auto_ptr<Player> CreateOGGPlayer(MainManager& mainManager)
{
    return std::auto_ptr<Player>(new VORBIS::OGGPlayer(mainManager));
}

std::auto_ptr<Player> CreateWMAPlayer(MainManager& mainManager)
{
    return std::auto_ptr<Player>(new FFMPEG::WMAPlayer(mainManager));
}

MainWnd::MainWnd(MainManager& mainManager)
    : mainManager_(mainManager),
    playerFactory_(new PlayerFactory),
    playlist_(NULL),
    randomPlayMenu_(gettext("_Random play"), true),
    repeatPlayMenu_(gettext("R_epeat play"), true),
    compactPlayMenu_(gettext("_Compact mode"), true),
    mainlistPlayMenu_(viewMenuGroup_, gettext("Main _list mode"), true),
    dirPlayMenu_(viewMenuGroup_, gettext("_Directory mode"), true),
    titleDisplayAlignment_(0.5, 0.5, 0.85, 1.0),
    seekScaleAlignment_(0.5, 0.5, 0.85, 1.0),
    seekScale_(0.0, 0.0, 1.0),
    playImgBtn_( PIXMAPSDIR "play.png" , PIXMAPSDIR "play_o.png", PIXMAPSDIR "play_p.png"),
    stopImgBtn_( PIXMAPSDIR "stop.png" , PIXMAPSDIR "stop_o.png", PIXMAPSDIR "stop_p.png"),
    prevImgBtn_( PIXMAPSDIR "prev.png" , PIXMAPSDIR "prev_o.png", PIXMAPSDIR "prev_p.png"),
    nextImgBtn_( PIXMAPSDIR "next.png" , PIXMAPSDIR "next_o.png", PIXMAPSDIR "next_p.png"),
    scrolledWindow_(NULL),
    // load store value?
    prevWidth_(600),
    prevHeight_(371),
    defaultWidth_(600),
    defaultHeight_(371),
    defaultCompactWidth_(450),
    defaultCompactHeight_(106),
    execThread_(NULL),
    inThread_(false), 
    inPlay_(false),
    breakPlay_(false),
    breakPlayForSetDriver_(false),
    randomPlay_(false),
    repeatPlay_(false),
    skipTimeUpdate_(false),
    listState_(LISTSTATE_EMPTY)
{
    // Do you have mpeg 1 layer 1 file? Anyway, i call it mp1.
    playerFactory_->Register("mp1", &mainManager_, &CreateMP3Player); 
    playerFactory_->Register("mp2", &mainManager_, &CreateMP3Player); 
    playerFactory_->Register("mp3", &mainManager_, &CreateMP3Player); 
    playerFactory_->Register("ogg", &mainManager_, &CreateOGGPlayer); 
    playerFactory_->Register("wma", &mainManager_, &CreateWMAPlayer); 
    
    set_title("Sagua");
    set_resizable(true);
    set_size_request(defaultWidth_, defaultHeight_);
    set_default_icon_from_file( PIXMAPSDIR "sagua.png" );

    add(mainBox_);
    Gtk::Menu_Helpers::MenuList* menuList = &fileMenu_.items();
    menuList->push_back( Gtk::Menu_Helpers::MenuElem(gettext("_Add files"),
		sigc::mem_fun(*this, &MainWnd::OnMenuFileAdd)) );
    menuList->push_back( Gtk::Menu_Helpers::MenuElem(gettext("Add a _directory"),
		sigc::mem_fun(*this, &MainWnd::OnMenuFileDirAdd)) );
    menuList->push_back( Gtk::Menu_Helpers::SeparatorElem());
    menuList->push_back( Gtk::Menu_Helpers::StockMenuElem(Gtk::Stock::QUIT,
		sigc::mem_fun(*this, &MainWnd::OnMenuQuit)) );
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem(gettext("_File"), fileMenu_) );

    menuList = &editMenu_.items();
    repeatPlayMenu_.signal_toggled().connect(
	    sigc::mem_fun(*this, &MainWnd::OnMenuRepeat) );
    randomPlayMenu_.signal_toggled().connect(
	    sigc::mem_fun(*this, &MainWnd::OnMenuRandom) );
    menuList->push_back( repeatPlayMenu_ );
    menuList->push_back( randomPlayMenu_ );
    menuList->push_back( Gtk::Menu_Helpers::SeparatorElem());
    menuList->push_back( Gtk::Menu_Helpers::StockMenuElem(Gtk::Stock::PREFERENCES,
		sigc::mem_fun(*this, &MainWnd::OnMenuPreference)) );
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem(gettext("_Edit"), editMenu_) );

    menuList = &viewMenu_.items();
    mainlistPlayMenu_.signal_activate().connect(
	    sigc::mem_fun(*this, &MainWnd::OnMenuViewMainList) );
    menuList->push_back(mainlistPlayMenu_);
    dirPlayMenu_.signal_activate().connect(
	    sigc::mem_fun(*this, &MainWnd::OnMenuViewDir) );
    menuList->push_back(dirPlayMenu_);
    menuList->push_back( Gtk::Menu_Helpers::SeparatorElem());
    compactPlayMenu_.signal_toggled().connect(
	    sigc::mem_fun(*this, &MainWnd::OnMenuCompactView) );
    menuList->push_back(compactPlayMenu_); 
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem(gettext("_View"), viewMenu_) );

    menuList = &helpMenu_.items();
    menuList->push_back( Gtk::Menu_Helpers::MenuElem(gettext("_About"),
		sigc::mem_fun(mainManager_, &MainManager::About)) );
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem(gettext("_Help"), helpMenu_) );

    mainBox_.pack_start(menuBar_, Gtk::PACK_SHRINK);

    titleDisplayAlignment_.set_padding(12, 12, 0, 0);
    titleDisplayAlignment_.set_size_request(-1, 52);
    titleDisplayAlignment_.add(displayLabel_);
    playerBox_.pack_start(titleDisplayAlignment_);
    timeDisplayBox_.pack_start(*manage(new Gtk::Label));
    timeDisplayBox_.pack_start(timeLabel_, Gtk::PACK_SHRINK);
    timeDisplayBox_.pack_start(totalTimeLabel_, Gtk::PACK_SHRINK);
    timeDisplayBox_.pack_start(*manage(new Gtk::Label));
    playerBox_.pack_start(timeDisplayBox_);
    seekScale_.set_draw_value(false);
    seekScale_.set_update_policy(Gtk::UPDATE_DELAYED);
    seekScale_.signal_adjust_bounds().connect(
	sigc::mem_fun(*this, &MainWnd::OnScaleSeek) );
    seekScaleAlignment_.add(seekScale_);
    playerBox_.pack_start(seekScaleAlignment_);

    playCtlBox_.pack_start(*manage(new Gtk::Label));
    prevImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnPrev) );
    playCtlBox_.pack_start(prevImgBtn_, Gtk::PACK_SHRINK);
    stopImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnStop) );
    playCtlBox_.pack_start(stopImgBtn_, Gtk::PACK_SHRINK);
    playImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnPlay) );
    playCtlBox_.pack_start(playImgBtn_, Gtk::PACK_SHRINK);
    nextImgBtn_.SignalPress().connect(
	    sigc::mem_fun(*this, &MainWnd::OnBtnNext) );
    playCtlBox_.pack_start(nextImgBtn_, Gtk::PACK_SHRINK);
    playCtlBox_.pack_start(*manage(new Gtk::Label));
    playerBox_.pack_start(playCtlBox_);

    mainBox_.pack_start(playerBox_, Gtk::PACK_SHRINK);
 
    show_all_children();
    
    LoadLastState();

    DisplayVisualPanel(false);

}

MainWnd::~MainWnd()
{
    if(inThread_) {
	inPlay_ = false;
	player_->Stop();
	execThread_->join();
    }
}

void MainWnd::FinishPlay()
{
    execThread_->join();
    inThread_ = false;

    seekScale_.set_value(0.0);

    if(breakPlayForSetDriver_) {
	breakPlayForSetDriver_ = false;
	mainManager_.AudioSetDriverFromPreference();
    }

    if(!inPlay_) {    
	DisplayVisualPanel(false);

    } else {

	if(breakPlay_) {
	    breakPlay_ = false;
	    OnBtnPlay();
	    
	} else {
	    OnBtnNext();
    
	    if(listState_ == LISTSTATE_LAST) {
		inPlay_ = false;
		DisplayVisualPanel(false);
	    }
	}
    }
}

void MainWnd::TimeUpdate()
{
    if(skipTimeUpdate_ || !inThread_)
	return;
	
    char timeStr_[18];
    long int seconds = player_->CurrentTime();
    
    snprintf(timeStr_, 18, "%02u:%02u", static_cast<unsigned int>(seconds / 60), static_cast<unsigned int>(seconds % 60)); 
    
    timeLabel_.set_label(timeStr_);
    seekScale_.set_value(seconds);
}

bool MainWnd::on_delete_event(GdkEventAny* ev)
{
    OnMenuQuit();

    return false;
}

bool MainWnd::on_window_state_event(GdkEventWindowState* ev)
{
    if(ev->new_window_state & GDK_WINDOW_STATE_ICONIFIED) {
	
	if(mainManager_.HideMainWndOnIconify())
	    mainManager_.MainWndIconifyEvent();
	
    } else {

	Gtk::Widget::on_window_state_event(ev); 	
    
    }

    return false;
}

void MainWnd::OnMenuFileAdd()
{
    Gtk::FileSelection fileDlg(gettext("Choose files to add"));
    fileDlg.set_transient_for(*this);
    fileDlg.set_select_multiple(true);
    fileDlg.hide_fileop_buttons();

    int result = fileDlg.run();
    fileDlg.hide();

    switch(result)
    {
	case(Gtk::RESPONSE_OK):
	    {
		std::vector<std::string>().swap(selectedFilesVector_);
		
		typedef std::vector<std::string>::const_iterator iter;
		selectedFilesVector_ = fileDlg.get_selections();
		iter end = selectedFilesVector_.end();

		for(iter i = selectedFilesVector_.begin(); i != end; ++i)
		{
		    if(playerFactory_->Exist(*i))
			playlist_->Add(*i);
		}

		playlist_->ForceUpdatePage();

		break;
	    }
	case(Gtk::RESPONSE_CANCEL):
	default:
	    {

		break;
	    }	
    }
}

void MainWnd::OnMenuFileDirAdd()
{
    DirBrowseDlg dirBrowseDlg;

    dirBrowseDlg.set_transient_for(*this); 

    int result = dirBrowseDlg.run();
    dirBrowseDlg.hide();

    switch(result)
    {
	case(Gtk::RESPONSE_OK):
	    {
		if(!is_realized())
		    realize();
		
		get_window()->set_cursor(Gdk::Cursor(Gdk::WATCH));
		while(Glib::MainContext::get_default()->iteration(false));
		    
		std::vector<std::string>().swap(selectedFilesVector_);
		GetFilenameFromDirToVector(dirBrowseDlg.SelectedDirectory(), selectedFilesVector_);

		typedef std::vector<std::string>::const_iterator iter;
		iter end = selectedFilesVector_.end();
		for(iter i = selectedFilesVector_.begin(); i != end; ++i)
		{
		    if(playerFactory_->Exist(*i))
			playlist_->Add(*i);
		}

		get_window()->set_cursor();

		playlist_->ForceUpdatePage();

		break;
	    }
	case(Gtk::RESPONSE_CANCEL):
	default:
	    {

		break;
	    }	
    }
}

void MainWnd::OnMenuQuit()
{
    OnBtnStop();

    mainManager_.Quit();
}

void MainWnd::OnMenuPreference()
{
    PrefWnd prefWnd(mainManager_);
    prefWnd.set_transient_for(*this);
    Gtk::Main::run(prefWnd);
}

void MainWnd::OnMenuCompactView()
{
    if(compactPlayMenu_.get_active()) {
	SetSkipTimeUpdate(true);
	get_size(prevWidth_, prevHeight_);
	timeDisplayBox_.hide();
	seekScaleAlignment_.hide();	
	viewBox_->hide();
	set_size_request(defaultCompactWidth_, defaultCompactHeight_);
	resize(defaultCompactWidth_, defaultCompactHeight_);
	set_resizable(false);
    } else {
	SetSkipTimeUpdate(false);
	if(inThread_)
	    TimeUpdate();
	
	timeDisplayBox_.show();
	seekScaleAlignment_.show();	
	viewBox_->show();
	set_size_request(defaultWidth_, defaultHeight_);
	resize(prevWidth_, prevHeight_);
	set_resizable(true);
    }
}

void MainWnd::OnMenuViewMainList()
{
    Tracer::Trace trace("MainWnd::OnMenuViewMainList");
       
    if(mainlistPlayMenu_.get_active()) {
	OnBtnStop();
	selectedFile_ = "";
	DisplayTitle("", true);

	if(viewBox_.get() != NULL)
	    mainBox_.remove(*viewBox_);

	viewBox_ = std::auto_ptr<Gtk::HBox>(new Gtk::HBox);

	scrolledWindow_ = manage(new Gtk::ScrolledWindow);
	
	int titleWidth, dummyHeight;
	if(compactPlayMenu_.get_active()) {
	    titleWidth = prevWidth_;
	    titleWidth -= 110;
	} else {
	    get_size(titleWidth, dummyHeight);
	    titleWidth -= 110;
	} 
	    
	playlist_ = manage(new Playlist(*this, "main", static_cast<unsigned int>(titleWidth)));
	scrolledWindow_->add(*playlist_);
	scrolledWindow_->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	viewBox_->pack_start(*scrolledWindow_);

	mainBox_.pack_start(*viewBox_);
	viewBox_->show();
	viewBox_->show_all_children();
    }
}

void MainWnd::OnMenuViewDir()
{
    Tracer::Trace trace("MainWnd::OnMenuViewDir");
    
    if(dirPlayMenu_.get_active()) {
	
	OnBtnStop();
	selectedFile_ = "";
	DisplayTitle("", true);

	if(viewBox_.get() != NULL)
	    mainBox_.remove(*viewBox_);

	viewBox_ = std::auto_ptr<Gtk::HBox>(new Gtk::HBox);

	int titleWidth, dummyHeight;
	if(compactPlayMenu_.get_active()) {
	    titleWidth = prevWidth_;
	    titleWidth -= 339;
	} else {
	    get_size(titleWidth, dummyHeight);
	    titleWidth -= 339;
	} 

	DirPlayWnd* dirPlayWnd = manage(new DirPlayWnd(*this, static_cast<unsigned int>(titleWidth)));
	playlist_ = dirPlayWnd->GetPlaylist();
	viewBox_->pack_start(*dirPlayWnd);

	mainBox_.pack_start(*viewBox_);
	viewBox_->show();
	viewBox_->show_all_children();
    }
}

void MainWnd::OnBtnPlay()
{
    Tracer::Trace trace("MainWnd::OnBtnPlay");
    
    if(selectedFile_.empty()) {
	selectedFile_ = playlist_->Get();

	if(listState_ == LISTSTATE_EMPTY) {
	    trace.Debug("Can't get any filename from list");
	    inPlay_ = false;
	    return;
	}
    }

    inPlay_ = true;

    if(inThread_) {
	trace.Debug("There exists another play thread");
	breakPlay_ = true;
	player_->Stop();
	return;
    }

    if(IsReadable(selectedFile_)) {
	player_ = playerFactory_->Create(selectedFile_);

	if(player_.get() == NULL) {
	    trace.Debug("Can't create player");
	    OnBtnNext();
	    return;
	}
    } else {
	trace.Debug("selectedFile_ is not readable");
	OnBtnNext();
	return;
    }

    if(player_->Init(selectedFile_)) {
	trace.Debug("Success initialize player");
	execThread_ = Glib::Thread::create(sigc::mem_fun(*player_, &Player::Start), true);
	inThread_ = true;
	DisplayVisualPanel(true);
    } else {
	trace.Debug("Can't initialize player"); 
	OnBtnNext();
    }
}

void MainWnd::OnBtnStop()
{
    if(inThread_) {
	inPlay_ = false;
	player_->Stop();
    }

}

void MainWnd::OnBtnPrev()
{
    if(!randomPlay_)
	selectedFile_ = playlist_->GetPrev();
    else
	selectedFile_ = playlist_->GetRandom();	

    if(inPlay_ && listState_ != LISTSTATE_FIRST)
	OnBtnPlay();

}

void MainWnd::OnBtnNext()
{
    if(!randomPlay_)
	selectedFile_ = playlist_->GetNext();
    else	
	selectedFile_ = playlist_->GetRandom();

    if(inPlay_ && listState_ != LISTSTATE_LAST)
	OnBtnPlay();
}

void MainWnd::OnScaleSeek(const double& newvalue)
{
    player_->Seek(static_cast<unsigned long>(newvalue));
}

void MainWnd::OnMenuRepeat()
{
    if(repeatPlayMenu_.get_active())
	repeatPlay_ = true;
    else
	repeatPlay_ = false;		
}

void MainWnd::OnMenuRandom()
{
    if(randomPlayMenu_.get_active())
	randomPlay_ = true;
    else
	randomPlay_ = false;		
}

void MainWnd::DisplayTitle(const Glib::ustring& title, const bool& clear)
{
    if(!clear) {
	
	Glib::ustring display = "<span weight=\"bold\" size=\"large\">";
	// title may be 32 character at maximum, and total length will 
	// be 72 at maximum. And after replace escape sequence, some more
	// length. So who does know exact length? 
	display.reserve(80);
	display += Glib::Markup::escape_text(title);
	display += "</span>";

	displayLabel_.set_label(display);
	displayLabel_.set_use_markup(true);
	displayLabel_.set_alignment(0.5, 0.5);

	set_title("Sagua - " + title);
	mainManager_.TrayIconSetTooltips(title);
	
    } else {
	
	displayLabel_.set_label("");
	set_title("Sagua");
	mainManager_.TrayIconSetTooltips("Sagua");

    }
}

void MainWnd::DisplayTotalTime(const Glib::ustring& time)
{
    totalTimeLabel_.set_label(" / " + time);
}

void MainWnd::SetSeekScaleRange(const unsigned int& max)
{
    seekScale_.set_range(0.0, static_cast<double>(max));
}

void MainWnd::LoadLastState()
{
    std::string lastStateFilename = Glib::get_home_dir();
    lastStateFilename += "/.sagua/laststate";

    std::map<std::string, std::string> configMap;
    // for directory entry, load configfile not lowercase!
    if(ConfigfileToMap(lastStateFilename, configMap, false, false)) {
	std::map<std::string, std::string>::const_iterator i, end;
	end = configMap.end();

	i = configMap.find("PlayRepeat");
	if(i != end) {
	    long int repeat = strtol( (i->second).c_str(), NULL, 10);
	    switch(repeat) {
		case 1:
		    repeatPlayMenu_.set_active(true);
		    break;
		    
		case 0:
		default:
		    repeatPlayMenu_.set_active(false);
	    }
	}

	i = configMap.find("PlayRandom");
	if(i != end) {
	    long int random = strtol( (i->second).c_str(), NULL, 10 );
	    switch(random) {
		case 1:
		    randomPlayMenu_.set_active(true);
		    break;
		        
		case 0:
		default:
		    randomPlayMenu_.set_active(false);
	    }
	}
	
	i = configMap.find("LastBrowseDirectory");
	if(i != end)
	    lastBrowsedDirectory_ = i->second;
	else
	    lastBrowsedDirectory_ = Glib::get_home_dir();  

	long int width = 0;
	i = configMap.find("WindowWidth");
	if(i != end)
	    width = strtol( (i->second).c_str(), NULL, 10);

	long int height = 0;
	i = configMap.find("WindowHeight");
	if(i != end)
	    height = strtol( (i->second).c_str(), NULL, 10);
	    
	if(width != 0 && height != 0) {
	    if(width > defaultWidth_ && height > defaultHeight_) {
		prevWidth_ = static_cast<int>(width);
		prevHeight_ = static_cast<int>(height);

		resize(prevWidth_, prevHeight_);
	    }
	}

	i = configMap.find("View");
	if(i != end) {
	    long int view = strtol( (i->second).c_str(), NULL, 10 );
	    switch(view) {
		case 0:
		    mainlistPlayMenu_.activate();
		    break;
		    
		case -1:
		    dirPlayMenu_.activate();
		    break;
		    
		default:
		    mainlistPlayMenu_.activate();
	    }
	}

	i = configMap.find("ViewCompact");
	if(i != end) {
	    long int compact = strtol( (i->second).c_str(), NULL, 10 );
	    switch(compact) {
		case 1:
		    compactPlayMenu_.set_active(true);
		    break;
		    
		case 0:
		default:
		    compactPlayMenu_.set_active(false);
	    }
	}
    } else {
	// else can't load config file to map	
	repeatPlayMenu_.set_active(false);
	randomPlayMenu_.set_active(false);
	lastBrowsedDirectory_ = Glib::get_home_dir();  
	mainlistPlayMenu_.activate();
	compactPlayMenu_.set_active(false);
    }
}

void MainWnd::CurrentFileInfo()
{
    if(playlist_)
	playlist_->ShowCurrentFileInfoDlg();
}

void MainWnd::SaveState()
{
    std::string programDir = Glib::get_home_dir();
    programDir += "/.sagua";

    if(!Glib::file_test(programDir, Glib::FILE_TEST_EXISTS)) {	
	// make dir with permission 755
	if(!mkdir(programDir.c_str(), S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH))
	    return;

    } else if(!Glib::file_test(programDir, Glib::FILE_TEST_IS_DIR)) 
	return;

    std::string lastStateFilename = programDir;
    lastStateFilename += "/laststate";

    std::ofstream lastStateFile(lastStateFilename.c_str());

    if(lastStateFile) {
	int lastWidth = 0;
	int lastHeight = 0;
	if(prevWidth_ != defaultWidth_ || prevHeight_ != defaultHeight_) {
	    lastWidth = prevWidth_;
	    lastHeight = prevHeight_;
	} 

	if(!compactPlayMenu_.get_active()) 
	    get_size(lastWidth, lastHeight);

	int lastViewWindow = 0;	    
	if(dirPlayMenu_.get_active())
	    lastViewWindow = -1;

	
	lastStateFile << "WindowWidth = " << lastWidth << std::endl;
	lastStateFile << "WindowHeight = " << lastHeight << std::endl;
	lastStateFile << "PlayRepeat = " << repeatPlayMenu_.get_active() << std::endl;
	lastStateFile << "PlayRandom = " << randomPlayMenu_.get_active() << std::endl;
	lastStateFile << "View = " << lastViewWindow << std::endl;
	lastStateFile << "ViewCompact = " << compactPlayMenu_.get_active() << std::endl;
	lastStateFile << "LastBrowseDirectory = " << lastBrowsedDirectory_ << std::endl;
    }
}
