// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <fstream>

#include "../include/misc.h"

using std::string;
using std::vector;

string GetExt(const string& filename)
{
    typedef string::const_iterator iter;

    iter begin = filename.begin();
    iter end = filename.end();
    iter coma = end;

    for(iter i = begin; i != end; ++i)
	if(*i == '.') 
	    coma = i;

    if(coma != end) {
	string temp(++coma, end);
	for_each(temp.begin(), temp.end(), ToLower());

	return temp;
    }
    else
	return string();	
}

// 'assume''s default argument is some magic constant 'eucKR' at function declartion.
// Really ass u me .... :(
Glib::ustring StringToUTF8(const Glib::ustring& orig, const std::string& fail, const std::string& assume)
{
    static std::string currentCharSet;
    static bool isCharSetUTF8;
    
    if(orig.validate()) 
	return orig;
    else {
	if(currentCharSet.empty())
	    isCharSetUTF8 = Glib::get_charset(currentCharSet);
	    
	if(isCharSetUTF8) {
	    try{
		return StringToUTF8( Glib::convert_with_fallback(orig, "UTF-8", assume), fail);
	    } catch (Glib::ConvertError& err) {
		if(fail.empty())
		    return Glib::ustring();
		else
		    return StringToUTF8(fail, "");    
	    }

	} else {
	    try {
		return StringToUTF8( Glib::convert_with_fallback(orig, "UTF-8", currentCharSet), fail);
	    } catch (Glib::ConvertError& err) {
		if(fail.empty())
		    return Glib::ustring();
		else    
		    return StringToUTF8(fail, "");
	    }
	    
	}
    } 
}

void SplitEachDirToVector(const string& fullpath, vector<string>& dest)
{
    typedef string::const_iterator iter;

    iter begin = fullpath.begin();
    iter end = fullpath.end();
    iter temp;
    while(begin != end)
    {
	begin = find_if(begin, end, std::not1(IsCharSlash()));
	temp = find_if(begin, end, IsCharSlash());
	if(begin != end)
	    dest.push_back(string(begin, temp));
	begin = temp;		
    }

}

void GetFilenameFromDirToVector(const std::string& path, std::vector<std::string>& dest, bool recursive)
{
    Glib::Dir dir(path);

    string fullpath;
    for(Glib::Dir::const_iterator eachFile = dir.begin(); eachFile != dir.end(); ++eachFile)
    {
	fullpath = path;
	fullpath += '/';
	fullpath += *eachFile;
	
	if(IsDirectory(fullpath)) {
	    if(recursive && IsExecutable(fullpath) && IsReadable(fullpath)) {
		GetFilenameFromDirToVector(fullpath, dest, recursive);
	    }
	} else {
	    dest.push_back(fullpath);
	}
    }

}

bool ConfigfileToMap(const std::string& filename, std::map<std::string, std::string>& configMap, const bool& convertLowerCase, const bool& valueBreakAtSpace)
{
    std::ifstream prefFile(filename.c_str());

    if(prefFile) {
	typedef std::string::iterator iter;
	std::string eachLine;

	iter i, j, k;
	std::string key;

	while(std::getline(prefFile, eachLine)) {
	    // Pass comment line, but there is no comment probably unless 
	    // edited by user. Because i don't write comment line to config file. :)
	    if(eachLine[0] == '#')
		continue;

	    // Convert string to lower case
	    if(convertLowerCase) 
		for_each(eachLine.begin(), eachLine.end(), ToLower());
	    
	    i = eachLine.begin();
	    // Eleminate front space character 
	    i = find_if(i, eachLine.end(), std::not1(IsCharSpace()));
	    
	    // Find '=' character
	    j = find_if(i, eachLine.end(), IsCharEqual());
	    if(i != eachLine.end() && i != j) {
		// Eleminate space character between i to j and that is key to insert. 
		k = find_if(i, j, IsCharSpace());
		key = std::string(i, k);
		
		// Find value to insert.
		i = ++j;
		i = find_if(i, eachLine.end(), std::not1(IsCharSpace()));

		if(valueBreakAtSpace)
		    j = find_if(i, eachLine.end(), IsCharSpace());
		else
		    j = eachLine.end();

		if(i != eachLine.end()) {
		    configMap[key] = std::string(i, j);
		}
	    }
	}
    } else
	return false;
    
    return true;
}
